<?php

namespace App\Http\Controllers\Auth;

use App\User;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Session;
use Validator;
use App\Http\Controllers\Controller;
use Illuminate\Foundation\Auth\ThrottlesLogins;
use Illuminate\Foundation\Auth\AuthenticatesAndRegistersUsers;
use Illuminate\Foundation\Auth\AuthenticatesUsers;
use Illuminate\Http\Request;

class AuthController extends Controller
{

    use AuthenticatesAndRegistersUsers;
    /*
    |--------------------------------------------------------------------------
    | Registration & Login Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles the registration of new users, as well as the
    | authentication of existing users. By default, this controller uses
    | a simple trait to add these behaviors. Why don't you explore it?
    |
    */

    use AuthenticatesAndRegistersUsers, ThrottlesLogins;

    /**
     * Where to redirect users after login / registration.
     *
     * @var string
     */
    protected $redirectTo = '/';
    protected $redirectAfterLogout = '/login';

    /**
     * Create a new authentication controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware($this->guestMiddleware(), ['except' => ['logout', 'update', 'changePassword', 'security']]);
    }

    /**
     * Get a validator for an incoming registration request.
     *
     * @param  array  $data
     * @return \Illuminate\Contracts\Validation\Validator
     */
    protected function validator(array $data)
    {
        return Validator::make($data, [
            'firstname' => 'required|max:255',
            'lastname' => 'required|max:255',
            'company_id' => 'required',
            'email' => 'required|email|max:255',
            'password' => 'required|min:6|confirmed',
        ]);
    }

    /**
     * Create a new user instance after a valid registration.
     *
     * @param  array  $data
     * @return User
     */
    protected function create(array $data)
    {
        return User::create([
            'firstname' => $data['firstname'],
            'lastname' => $data['lastname'],
            'name' => $data['firstname'].$data['lastname'],
            'email' => $data['email'],
            'company_id' => $data['company_id'],
            'phone' => $data['phone'],
            'status' => $data['status'],
//            'country_id' => $data['country_id'],
//            'country_name' => $data['country_name'],
//            'state_id' => $data['state_id'],
//            'state_name' => $data['state_name'],
//            'city_id' => $data['city_id'],
//            'city_name' => $data['city_name'],
            'password' => bcrypt($data['password']),
        ]);
    }


    protected function validateLogin(Request $request)
    {
        $messages = [
            'required' => 'The :attribute field is required.',
        ];

//        $this->validate($request, [
//            'email' => 'required',
//            'password' => 'required',
//            'company_id' => 'required|exists:users,company_id,'.'company_id,'.config('app.id'),
//        ]);

        $valid = Validator::make($request->all(), [
            'email' => 'required',
            'password' => 'required',
            'company_id' => 'required|exists:users,company_id,'.'company_id,'.config('app.id'),
        ], $messages);

        if($valid->fails()){
            return back()->withErrors($valid);
        }
    }

    public function update(Request $request){
        $valid = Validator::make($request->all(), [
            'firstname' => 'required',
            'lastname' => 'required',
            'phone' => 'required'
        ]);
        if($valid->fails()){
            return back()->withErrors($valid);
        }
        try{
            $member = new \App\User();
            $member::where('id', Auth::user()->id)->update([
                'firstname' => $request->firstname,
                'lastname' => $request->lastname,
                'phone' => $request->phone,
                'address' => trim($request->address)
            ]);
            if($member){
                return back()->withErrors("Profile Updated.");
            }
        }catch (\Exception $e){
            return back()->withErrors("Error Updating Profile.". $e->getMessage());
        }
    }

    public function changePassword(){
        return view('security.changepassword');
    }


    public function security(){
        return view('security.index');
    }

    protected function showLoginForm(){
        return view('security.index');
    }

    protected function showResetForm (){
        return view('security.changepassword');
    }
}
